/**
 * Behavior for 'aggrid_widget_type' widget.
 */
(function ($) {
    'use strict';

    var aggridFieldName = new Array();
    var aggridLicense = new Array();
    var aggridDataEdit = new Array();

    var eGridDiv = new Array();
    var gridOptions = new Array();

    Drupal.aggridInstances = [];

    Drupal.behaviors.aggridIntegration = {
        attach: function (context) {
            function makeJson(item) {
                var aggridOutput = {};
                aggridOutput = gridOptions[item].rowData;

                // Write back to Drupal "Value" field
                $('#' + aggridFieldName[item] +'_rowData').val(JSON.stringify(aggridOutput));
            }

            function formatNumber(number) {
                // this puts commas into the number eg 1000 goes to 1,000,
                // i pulled this from stack overflow, i have no idea how it works
                return Math.floor(number).toString().replace(/(\d)(?=(\d{3})+(?!\d))/g, "$1,");
            }

            // @todo Init functionality should support multiple drop zones on page.
            var selector = $('.aggrid-widget');
            var idArray = [];

            selector.each(function () {
                idArray.push(this.id);
            });

            jQuery.each(idArray, function(index, item) {
                aggridFieldName[item] = $("#"+ item).data("target");
                aggridLicense = $("#"+ item).data("license");
                aggridDataEdit[item] = $("#"+ item).data("edit");

                $("#"+ item)
                    .addClass('aggrid')
                    .css('height', '500px');

                // If DropzoneJS instance is already registered on Element. There is no
                // need to register it again.
                if ($("#"+ item).once("#"+ item).length !== $("#"+ item).length) {
                    return;
                }

                // Get the data
                var colDefsValue = $('#' + aggridFieldName[item] +'_columnDefs').val();
                var rowSettingsValue = $('#' + aggridFieldName[item] +'_rowSettings').val();
                var rowDataValue = $('#' + aggridFieldName[item] +'_rowData').val();
                var addOptValue = $('#' + aggridFieldName[item] +'_addOptions').val();

                // If it's not blank, parse the json... otherwise, null
                var aggridJSON_colDefs = colDefsValue ? JSON.parse(colDefsValue) : null;
                var aggridJSON_rowSettings = rowSettingsValue ? JSON.parse(rowSettingsValue) : null;
                var aggridJSON_rowData = rowDataValue ? JSON.parse(rowDataValue) : null;
                var aggridJSON_addOpt = addOptValue ? JSON.parse(addOptValue) : null;

                function ParseChildren(data)
                {
                    // Add column span ability to each header. It will span a cell if colspan = ##
                    var aggridJSON_Span;

                    var result = '';

                    for (var i=0; i< data.length; i++)
                    {
                        result += data[i].headerName;
                        result += ", ";

                        if (typeof data[i].field !== 'undefined') {
                            aggridJSON_Span = {
                                editable: function(params) {if(typeof eval('aggridJSON_rowSettings[' + params.node.id + '].editable_' + params.column.colId + '') === 'boolean') {return eval('aggridJSON_rowSettings[' + params.node.id + '].colSpan_' + params.column.colId  + '');} else if (typeof eval('aggridJSON_rowSettings[' + params.node.id + '].editable_allRow') === 'boolean') {return eval('aggridJSON_rowSettings[' + params.node.id + '].colSpan_allRow');} else {return 1;}},
                                colSpan: function(params) {if($.isNumeric(eval('aggridJSON_rowSettings[' + params.node.id + '].colSpan_' + params.column.colId + ''))) {return eval('aggridJSON_rowSettings[' + params.node.id + '].colSpan_' + params.column.colId  + '');} else {return 1;}},
                                rowSpan: function(params) {if($.isNumeric(eval('aggridJSON_rowSettings[' + params.node.id + '].rowSpan_' + params.column.colId + ''))) {return eval('aggridJSON_rowSettings[' + params.node.id + '].rowSpan_' + params.column.colId  + '');} else {return 1;}}
                            };
                            data[i] = $.extend(data[i], aggridJSON_Span);
                        }

                        if (data[i].children && data[i].children.length > 0)
                            result += ParseChildren(data[i].children);

                    }
                }

                // Only parse all headers/children if rowSettings is available
                if (aggridJSON_rowSettings !== null) {
                    ParseChildren(aggridJSON_colDefs);
                }

                // Build JSON for ag-Grid
                var aggridJSON = {
                    "columnDefs": aggridJSON_colDefs,
                    "rowData": aggridJSON_rowData
                };

                aggridJSON = $.extend(aggridJSON, aggridJSON_addOpt);

                // Default Options for all ag-Grid
                var default_gridOptions = {
                    stopEditingWhenGridLosesFocus: true,
                    defaultColDef: {
                        editable: aggridDataEdit[item],
                    },
                    onGridReady: function () {
                        gridOptions[item].api.sizeColumnsToFit();
                    },
                    onCellValueChanged: function() {
                        makeJson(item);
                    }
                };

                //colDef.menuTabs: ['filterMenuTab','columnsMenuTab'],

                // Add the Default Options
                gridOptions[item] = aggridJSON;
                gridOptions[item] = $.extend(aggridJSON, default_gridOptions);

                // Apply the license if it is available
                if (aggridLicense === '') {
                } else {
                    agGrid.LicenseManager.setLicenseKey(aggridLicense);
                }

                // Get the ag-Grid Div and start it up
                eGridDiv[item] = document.querySelector("#" + item);

                // create the grid passing in the div to use together with the columns & data we want to use
                new agGrid.Grid(eGridDiv[item], gridOptions[item]);

            });
        }
    }

})(jQuery);
